"""Application container and component definitions."""

from __future__ import annotations

from .components import component
from .container import AppContainer

__all__ = ["AppContainer", "component"]
